#include "server.hpp"

namespace xmlioserver {
namespace comm {

   /// ////////////////////// Définitions ////////////////////// ///
   
   CServer::CServer(MPIComm comm_client_server)
      : blist(comm_client_server)
   { /* Ne rien faire de plus */ }  
        
   CServer::~CServer(void)
   { /* Ne rien faire de plus */ } 
   
   ///--------------------------------------------------------------
   
   const CBufferList & CServer::getBufferList(void) const 
   {
      return (this->blist);
   }   
   
   //---------------------------------------------------------------
   
   void CServer::run(void)
   {
      while (this->blist.recvRequests())
      {
         long int managerId = 0, methodId  = 0, nbargs  = 0;
         long int managerId_= 0, methodId_ = 0, nbargs_ = 0;
         std::vector<CLinearBuffer> lbuffer;
         this->blist.getRequests(lbuffer);
         for (StdSize i = 0; i < lbuffer.size(); i++)
         {
            lbuffer[i].getRequestInfos(0, managerId, methodId, nbargs);
            if (((managerId_ != managerId) || (methodId_ != methodId) || (nbargs_ != nbargs)) && (i != 0 ))
            {
               /*std::cout << managerId_ << "<->" << managerId << std::endl
                           << methodId_  << "<->" << methodId  << std::endl
                           << nbargs_    << "<->" << nbargs    << std::endl;*/
               ERROR("CServer::run(void)", << "[" << i << "] Les requêtes ne sont pas synchronisées !");
            }
            managerId_ = managerId;
            methodId_  = methodId;
            nbargs_    = nbargs;
         }
            
         if (managerId == 0)
         {
            switch(methodId)
            {
               case (0) :
                  this->initialize();
                  continue;
               case (1) :
                  this->finalize();
                  return;
               default  :
                  ERROR("CServer::run(void)", 
                     << "[ managerId = " << managerId << ", "
                     << "[ methodId = "  << methodId  << ", "
                     << "[ nbargs = "    << nbargs    << "] "
                     << " Methode inconnue !");
            }
         }
         if (managerId == 1)
         {
            switch(methodId)
            {
               case (0) :
                  this->setContext(lbuffer);
                  continue;
               case (1) :
                  this->updateCalendar(lbuffer);
                  continue;
               default  :
                  ERROR("CServer::run(void)", 
                     << "[ managerId = " << managerId << ", "
                     << "[ methodId = "  << methodId  << ", "
                     << "[ nbargs = "    << nbargs    << "] "
                     << " Methode inconnue !");
            }
         }
         if (managerId == 2)
         {
            switch(methodId)
            {
               case (0) :
                  this->writeData(lbuffer, 4);
                  continue;
               case (1) :
                  this->writeData(lbuffer, 8);
                  continue;
               default  :
                  ERROR("CServer::run(void)", 
                     << "[ managerId = " << managerId << ", "
                     << "[ methodId = "  << methodId  << ", "
                     << "[ nbargs = "    << nbargs    << "] "
                     << " Methode inconnue !");
            }
         }
         ERROR("CServer::run(void)", 
                << "[ managerId = " << managerId << ", "
                << "[ methodId = "  << methodId  << ", "
                << "[ nbargs = "    << nbargs    << "] "
                << " Methode inconnue !");
      }
   }

   //--------------------------------------------------------------
   
   void CServer::initialize(void) // manager 0, method 0
   {
      std::cout << "initialize called " << std::endl;
   }
   
   //--------------------------------------------------------------
   
   void CServer::finalize(void) // manager 0, method 1
   {
      std::cout << "finalize called " << std::endl;
   }
   
   
   //--------------------------------------------------------------
             
   void CServer::setContext(const std::vector<CLinearBuffer> & buffer) // manager 1, method 0
   {
      std::cout << "setContext called " << std::endl;
   }
   
   //--------------------------------------------------------------
   
   void CServer::updateCalendar(const std::vector<CLinearBuffer> & buffer) // manager 1, method 1
   {
      std::cout << "updateCalendar called " << std::endl;
   }
   
   //--------------------------------------------------------------
         
   void CServer::writeData(const std::vector<CLinearBuffer> & buffer, int prec)  // manager 2, method 0 - 1
   {
      std::cout << "writeData called " << std::endl;
   }
  
   ///--------------------------------------------------------------
   
   
} // namespace comm
} // namespace xmlioserver
